@@ -1,5 +1,5 @@
 package JSON::Lines;
-use 5.006; use strict; use warnings; our $VERSION = '0.02';
+use 5.006; use strict; use warnings; our $VERSION = '0.03';
 use JSON; use base 'Import::Export';
 
 our ($JSON, $LINES, %EX);
@@ -46,8 +46,8 @@ sub encode {
 				die $@;
 			}
 		} else {
-			$self->{success_cb}->($json) if $self->{success_cb};
-			$stream .= ($stream && $stream !~ m/\n$/ ? "\n" : "") .  $json;
+			$self->{success_cb}->($json, $_) if $self->{success_cb};
+			$stream .= $json . ($json =~ m/\n$/ ? "" : "\n");
 		}
 	}
 	return $self->{stream} = $stream;
@@ -64,19 +64,8 @@ sub encode_file {
 sub decode {
 	my ($self, $string) = @_;
 	my @lines;
-	for ($string =~ m/$LINES/g) {
-		my $struct = eval { $JSON->decode($_) };
-		if ($@) {
-			if ($self->{error_cb}) {
-				$self->{error_cb}->($@, $_);
-			} else {
-				die $@;
-			}
-		} else {
-			$self->{success_cb}->($struct) if $self->{success_cb};
-			push @lines, $struct;
-		}
-	}
+	push @lines, $self->_decode_line($_) 
+		for ($string =~ m/$LINES/g);
 	@lines = $self->_deparse_headers(@lines)
 		if ($self->{parse_headers});
 	return wantarray ? @lines : \@lines;
@@ -91,16 +80,53 @@ sub decode_file {
 }
 
 sub add_line {
-	my $stream = $_[0]->{stream};
-	my $add = $_[0]->encode([$_[1]]);
-	$_[0]->{stream} = ($stream ? "$stream\n" : "") . $add;
-	$_[0]->{stream};
+	my ($self, $line, $fh) = @_;
+	if (defined $fh) {
+		print $fh $self->encode([$line]);
+	} else {
+		my $stream = $self->{stream};
+		my $add = $self->encode([$line]);
+		$self->{stream} =  $stream .  $add;
+		$self->{stream};
+	}
 }
 
 sub clear_stream {
 	$_[0]->{stream} = '';
 }
 
+sub get_lines {
+	my ($self, $fh, $lines) = @_;
+	my @lines;
+	for (1 .. $lines) {
+		my $line = $self->get_line($fh);
+		push @lines, $line;
+		last if eof($fh);
+	}
+	return wantarray ? @lines : \@lines;
+}
+
+sub get_line {
+	my ($self, $fh) = @_;
+	my $line = '';
+	$line .= <$fh> while ($line !~ m/^$LINES/ && !eof($fh));
+	return $self->_decode_line($line);
+}
+
+sub _decode_line {
+	my ($self, $line) = @_;
+	my $struct = eval { $JSON->decode($line) };
+	if ($@) {
+		if ($self->{error_cb}) {
+			return $self->{error_cb}->($@, $line);
+		} else {
+			die $@;
+		}
+	}
+	return $self->{success_cb}->($struct, $line) if $self->{success_cb};
+	return $struct;
+}
+
 sub _parse_headers {
 	my ($self, @data) = @_;
 	my @headers = @{ $self->{headers} };
@@ -156,7 +182,7 @@ JSON::Lines - Parse JSONLines with perl.
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =cut
 
@@ -178,6 +204,8 @@ Quick summary of what the module does.
 
 	my $string = $jsonl->encode(@data);
 
+	my $file = $jsonl->encode_file('score.jsonl', @data);
+	
 	# ["Name", "Session", "Score", "Completed"]
 	# ["Gilbert", "2013", 24, true]
 	# ["Alexa", "2013", 29, true]
@@ -186,6 +214,20 @@ Quick summary of what the module does.
 
 	...
 
+	my $all = $jsonl->decode_file($file);
+
+	open my $fh, '<', $file or die $!;
+	while (my $line = $jsonl->get_line($fh)) {
+		push @lines, $line;
+	}
+	close $fh;
+
+	open my $fh, '<', $file or die $!;
+	my @hundred_lines = $jsonl->get_lines($fh, 100);
+	close $fh;
+
+	...
+
 	use JSON::Lines qw/jsonl/;
 
 	my $data = [
@@ -221,6 +263,7 @@ Quick summary of what the module does.
 	# {"name": "Alexa", "wins": [["two pair", "4♠"], ["two pair", "9♠"]]}
 	# {"name": "May", "wins": []}
 	# {"name": "Deloise", "wins": [["three of a kind", "5♣"]]}
+
 	
 =head1 DESCRIPTION
 
@@ -274,6 +317,24 @@ Add a new line to the current JSON::Lines stream.
 
 	$jsonl->add_line($line);
 
+	$jsonl->add_line($line, $fh);
+
+=head2 get_line
+
+Decode a json lines file, one line at a time.
+
+	open my $fh, "<", $file or die "$file: $!";
+	my $line = $jsonl->get_line($fh);
+	close $fh;
+
+=head2 get_lines
+
+Decode a json lines file, 'n' lines at a time.
+
+	open my $fh, "<", $file or die "$file: $!";
+	my @lines = $jsonl->get_lines($fh, 100);
+	close $fh;
+
 =head2 clear_stream
 
 Clear the current JSON::Lines stream.
